@@ -4,6 +4,7 @@ mod untyped;
 
 #[cfg(test)]
 mod tests;
+pub mod visit;
 
 pub use self::typed::TypedExpr;
 pub use self::untyped::{UntypedExpr, Use};
@@ -16,6 +17,7 @@ use crate::type_::expression::Implementations;
 use crate::type_::{
     self, Deprecation, ModuleValueConstructor, PatternConstructor, Type, ValueConstructor,
 };
+use std::cmp::Ordering;
 use std::sync::Arc;
 
 use ecow::EcoString;
@@ -347,6 +349,63 @@ impl TypeAst {
             },
         }
     }
+
+    pub fn find_node(&self, byte_index: u32, type_: Arc<Type>) -> Option<Located<'_>> {
+        if !self.location().contains(byte_index) {
+            return None;
+        }
+
+        match self {
+            TypeAst::Fn(TypeAstFn {
+                arguments, return_, ..
+            }) => type_
+                .fn_types()
+                .and_then(|(arg_types, ret_type)| {
+                    if let Some(arg) = arguments
+                        .iter()
+                        .zip(arg_types)
+                        .find_map(|(arg, arg_type)| arg.find_node(byte_index, arg_type.clone()))
+                    {
+                        return Some(arg);
+                    }
+                    if let Some(ret) = return_.find_node(byte_index, ret_type) {
+                        return Some(ret);
+                    }
+
+                    None
+                })
+                .or(Some(Located::Annotation(self.location(), type_))),
+            TypeAst::Constructor(TypeAstConstructor { arguments, .. }) => type_
+                .constructor_types()
+                .and_then(|arg_types| {
+                    if let Some(arg) = arguments
+                        .iter()
+                        .zip(arg_types)
+                        .find_map(|(arg, arg_type)| arg.find_node(byte_index, arg_type.clone()))
+                    {
+                        return Some(arg);
+                    }
+
+                    None
+                })
+                .or(Some(Located::Annotation(self.location(), type_))),
+            TypeAst::Tuple(TypeAstTuple { elems, .. }) => type_
+                .tuple_types()
+                .and_then(|elem_types| {
+                    if let Some(e) = elems
+                        .iter()
+                        .zip(elem_types)
+                        .find_map(|(e, e_type)| e.find_node(byte_index, e_type.clone()))
+                    {
+                        return Some(e);
+                    }
+
+                    None
+                })
+                .or(Some(Located::Annotation(self.location(), type_))),
+            TypeAst::Var(_) | TypeAst::Hole(_) => Some(Located::Annotation(self.location(), type_)),
+        }
+    }
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -574,15 +633,27 @@ impl TypedDefinition {
     pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
         match self {
             Definition::Function(function) => {
+                // Search for the corresponding node inside the function
+                // only if the index falls within the function's full location.
+                if !function.full_location().contains(byte_index) {
+                    return None;
+                }
+
                 if let Some(found) = function.body.iter().find_map(|s| s.find_node(byte_index)) {
                     return Some(found);
-                };
+                }
 
                 if let Some(found_arg) = function
                     .arguments
                     .iter()
                     .find(|arg| arg.location.contains(byte_index))
                 {
+                    // Check if location is within the arg annotation.
+                    if let Some(a) = &found_arg.annotation {
+                        return a
+                            .find_node(byte_index, found_arg.type_.clone())
+                            .or(Some(Located::Arg(found_arg)));
+                    }
                     return Some(Located::Arg(found_arg));
                 };
 
@@ -594,6 +665,15 @@ impl TypedDefinition {
                     return Some(Located::Statement(found_statement));
                 };
 
+                // Check if location is within the return annotation.
+                if let Some(l) = function
+                    .return_annotation
+                    .iter()
+                    .find_map(|a| a.find_node(byte_index, function.return_type.clone()))
+                {
+                    return Some(l);
+                };
+
                 // Note that the fn `.location` covers the function head, not
                 // the entire statement.
                 if function.location.contains(byte_index) {
@@ -606,6 +686,23 @@ impl TypedDefinition {
             }
 
             Definition::CustomType(custom) => {
+                // Check if location is within the type of one of the arguments of a constructor.
+                if let Some(annotation) = custom
+                    .constructors
+                    .iter()
+                    .find(|constructor| constructor.location.contains(byte_index))
+                    .and_then(|constructor| {
+                        constructor
+                            .arguments
+                            .iter()
+                            .find(|arg| arg.location.contains(byte_index))
+                    })
+                    .filter(|arg| arg.location.contains(byte_index))
+                    .and_then(|arg| arg.ast.find_node(byte_index, arg.type_.clone()))
+                {
+                    return Some(annotation);
+                }
+
                 // Note that the custom type `.location` covers the function
                 // head, not the entire statement.
                 if custom.full_location().contains(byte_index) {
@@ -615,8 +712,66 @@ impl TypedDefinition {
                 }
             }
 
-            Definition::TypeAlias(_) | Definition::Import(_) | Definition::ModuleConstant(_) => {
+            Definition::TypeAlias(alias) => {
+                // Check if location is within the type being aliased.
+                if let Some(l) = alias.type_ast.find_node(byte_index, alias.type_.clone()) {
+                    return Some(l);
+                }
+
+                if alias.location.contains(byte_index) {
+                    Some(Located::ModuleStatement(self))
+                } else {
+                    None
+                }
+            }
+
+            Definition::ModuleConstant(constant) => {
+                // Check if location is within the annotation.
+                if let Some(annotation) = &constant.annotation {
+                    if let Some(l) = annotation.find_node(byte_index, constant.type_.clone()) {
+                        return Some(l);
+                    }
+                }
+
+                if constant.location.contains(byte_index) {
+                    Some(Located::ModuleStatement(self))
+                } else {
+                    None
+                }
+            }
+
+            Definition::Import(import) => {
                 if self.location().contains(byte_index) {
+                    if let Some(unqualified) = import
+                        .unqualified_values
+                        .iter()
+                        .find(|i| i.location.contains(byte_index))
+                    {
+                        return Some(Located::UnqualifiedImport(
+                            crate::build::UnqualifiedImport {
+                                name: &unqualified.name,
+                                module: &import.module,
+                                is_type: false,
+                                location: &unqualified.location,
+                            },
+                        ));
+                    }
+
+                    if let Some(unqualified) = import
+                        .unqualified_types
+                        .iter()
+                        .find(|i| i.location.contains(byte_index))
+                    {
+                        return Some(Located::UnqualifiedImport(
+                            crate::build::UnqualifiedImport {
+                                name: &unqualified.name,
+                                module: &import.module,
+                                is_type: true,
+                                location: &unqualified.location,
+                            },
+                        ));
+                    }
+
                     Some(Located::ModuleStatement(self))
                 } else {
                     None
@@ -1166,6 +1321,16 @@ impl SrcSpan {
     pub fn contains(&self, byte_index: u32) -> bool {
         byte_index >= self.start && byte_index < self.end
     }
+
+    pub fn cmp_byte_index(&self, byte_index: u32) -> Ordering {
+        if byte_index < self.start {
+            Ordering::Less
+        } else if self.end <= byte_index {
+            Ordering::Greater
+        } else {
+            Ordering::Equal
+        }
+    }
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -1433,23 +1598,19 @@ pub enum AssignmentKind {
     // let x = ...
     Let,
     // let assert x = ...
-    Assert,
+    Assert { location: SrcSpan },
 }
 
 impl AssignmentKind {
-    pub(crate) fn performs_exhaustiveness_check(&self) -> bool {
-        match self {
-            AssignmentKind::Let => true,
-            AssignmentKind::Assert => false,
-        }
-    }
-
     /// Returns `true` if the assignment kind is [`Assert`].
     ///
     /// [`Assert`]: AssignmentKind::Assert
     #[must_use]
     pub fn is_assert(&self) -> bool {
-        matches!(self, Self::Assert)
+        match self {
+            Self::Assert { .. } => true,
+            Self::Let => false,
+        }
     }
 }
 
@@ -1616,7 +1777,7 @@ pub enum TodoKind {
 
 #[derive(Debug, Default)]
 pub struct GroupedStatements {
-    pub functions: Vec<Function<(), UntypedExpr>>,
+    pub functions: Vec<UntypedFunction>,
     pub constants: Vec<UntypedModuleConstant>,
     pub custom_types: Vec<CustomType<()>>,
     pub imports: Vec<Import<()>>,
@@ -1682,6 +1843,14 @@ impl<T, E> Statement<T, E> {
     pub fn is_expression(&self) -> bool {
         matches!(self, Self::Expression(..))
     }
+
+    #[must_use]
+    pub(crate) fn is_use(&self) -> bool {
+        match self {
+            Self::Use(_) => true,
+            _ => false,
+        }
+    }
 }
 
 impl UntypedStatement {
@@ -1786,6 +1955,11 @@ pub type UntypedAssignment = Assignment<(), UntypedExpr>;
 
 impl TypedAssignment {
     pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
+        if let Some(annotation) = &self.annotation {
+            if let Some(l) = annotation.find_node(byte_index, self.pattern.type_()) {
+                return Some(l);
+            }
+        }
         self.pattern
             .find_node(byte_index)
             .or_else(|| self.value.find_node(byte_index))
