@@ -74,19 +74,18 @@ pub fn compile_package(
         let line_numbers = LineNumbers::new(dep_src);
         let mut config = PackageConfig::default();
         config.name = dep_package.into();
-        let dep = crate::analyse::infer_module::<()>(
-            Target::Erlang,
-            &ids,
-            ast,
-            Origin::Src,
-            &modules,
-            &TypeWarningEmitter::null(),
-            &std::collections::HashMap::new(),
-            TargetSupport::Enforced,
-            line_numbers,
-            &config,
-            "".into(),
-        )
+
+        let dep = crate::analyse::ModuleAnalyzerConstructor::<()> {
+            target: Target::Erlang,
+            ids: &ids,
+            origin: Origin::Src,
+            importable_modules: &modules,
+            warnings: &TypeWarningEmitter::null(),
+            direct_dependencies: &std::collections::HashMap::new(),
+            target_support: TargetSupport::Enforced,
+            package_config: &config,
+        }
+        .infer_module(ast, line_numbers, "".into())
         .expect("should successfully infer");
         let _ = modules.insert(dep_name.into(), dep.type_info);
         let _ = direct_dependencies.insert(dep_package.into(), ());
@@ -101,19 +100,17 @@ pub fn compile_package(
     ast.name = module_name.clone();
     let mut config = PackageConfig::default();
     config.name = "my_package".into();
-    let ast = crate::analyse::infer_module::<()>(
-        Target::Erlang,
-        &ids,
-        ast,
-        Origin::Src,
-        &modules,
-        &TypeWarningEmitter::null(),
-        &direct_dependencies,
-        TargetSupport::Enforced,
-        LineNumbers::new(src),
-        &config,
-        "".into(),
-    )
+    let ast = crate::analyse::ModuleAnalyzerConstructor {
+        target: Target::Erlang,
+        ids: &ids,
+        origin: Origin::Src,
+        importable_modules: &modules,
+        warnings: &TypeWarningEmitter::null(),
+        direct_dependencies: &direct_dependencies,
+        target_support: TargetSupport::Enforced,
+        package_config: &config,
+    }
+    .infer_module(ast, LineNumbers::new(src), "".into())
     .expect("should successfully infer");
 
     // TODO: all the bits above are basically copy pasted from the javascript
