@@ -244,8 +244,7 @@ impl<'a> Environment<'a> {
         );
     }
 
-    /// Insert a value into the current module.
-    /// Errors if the module already has a value with that name.
+    /// Insert (or overwrites) a value into the current module.
     ///
     pub fn insert_module_value(&mut self, name: EcoString, value: ValueConstructor) {
         let _ = self.module_values.insert(name, value);
@@ -334,6 +333,7 @@ impl<'a> Environment<'a> {
                         name: name.clone(),
                         module_name: module.name.clone(),
                         type_constructors: module.public_type_names(),
+                        imported_type_as_value: false,
                     })
             }
         }?;
@@ -380,6 +380,7 @@ impl<'a> Environment<'a> {
                         name: name.clone(),
                         module_name: module.name.clone(),
                         type_constructors: module.public_type_names(),
+                        imported_type_as_value: false,
                     }
                 })
             }
@@ -416,6 +417,7 @@ impl<'a> Environment<'a> {
                         name: name.clone(),
                         module_name: module.name.clone(),
                         value_constructors: module.public_value_names(),
+                        imported_value_as_type: false,
                     }
                 })
             }
@@ -563,7 +565,7 @@ impl<'a> Environment<'a> {
         }
 
         for (name, info) in self.unused_module_aliases.iter() {
-            if self.unused_modules.get(name).is_none() {
+            if !self.unused_modules.contains_key(name) {
                 self.warnings.emit(Warning::UnusedImportedModuleAlias {
                     alias: name.clone(),
                     location: info.location,
@@ -723,19 +725,18 @@ pub fn unify(t1: Arc<Type>, t2: Arc<Type>) -> Result<(), UnifyError> {
                 retrn: retrn2,
                 ..
             },
-        ) if args1.len() == args2.len() => {
-            for (a, b) in args1.iter().zip(args2) {
-                unify(a.clone(), b.clone()).map_err(|_| UnifyError::CouldNotUnify {
-                    expected: t1.clone(),
-                    given: t2.clone(),
-                    situation: None,
-                })?;
+        ) => {
+            if args1.len() != args2.len() {
+                Err(unify_wrong_arity(&t1, args1.len(), &t2, args2.len()))?
             }
-            unify(retrn1.clone(), retrn2.clone()).map_err(|_| UnifyError::CouldNotUnify {
-                expected: t1.clone(),
-                given: t2.clone(),
-                situation: None,
-            })
+
+            for (i, (a, b)) in args1.iter().zip(args2).enumerate() {
+                unify(a.clone(), b.clone())
+                    .map_err(|_| unify_wrong_arguments(&t1, a, &t2, b, i))?;
+            }
+
+            unify(retrn1.clone(), retrn2.clone())
+                .map_err(|_| unify_wrong_returns(&t1, retrn1, &t2, retrn2))
         }
 
         _ => Err(UnifyError::CouldNotUnify {
