@@ -344,7 +344,7 @@ fn case_tuple_guard() {
 
 #[test]
 fn case_list_guard() {
-    assert_error!("case [1] { x if x == [1, 2.0] -> 1 }");
+    assert_error!("case [1] { x if x == [1, 2.0] -> 1 _ -> 2 }");
 }
 
 #[test]
@@ -611,7 +611,7 @@ pub type Shape {
     Rectangle(x: String, y: String)
 }
 pub fn get_x(shape: Shape) { shape.x }
-pub fn get_y(shape: Shape) { shape.y }"
+"
     );
 }
 
@@ -911,7 +911,7 @@ fn guard_record_wrong_arity() {
     assert_module_error!(
         r#"type X { X(a: Int, b: Float) }
 fn x() {
-  case X(1, 2.0) { x if x == X(1) -> 1 }
+  case X(1, 2.0) { x if x == X(1) -> 1 _ -> 2 }
 }"#
     );
 }
@@ -920,7 +920,7 @@ fn x() {
 fn subject_int_float_guard_tuple() {
     assert_module_error!(
         r#"type X { X(a: Int, b: Float) }
-fn x() { case X(1, 2.0) { x if x == X(2.0, 1) -> 1 } }"#
+fn x() { case X(1, 2.0) { x if x == X(2.0, 1) -> 1 _ -> 2 } }"#
     );
 }
 
@@ -1160,6 +1160,74 @@ fn const_annotation_wrong_4() {
     assert_module_error!("pub const pair: #(Int, Float) = #(4.1, 1)");
 }
 
+#[test]
+fn const_multiple_errors_mismatched_types() {
+    assert_module_error!(
+        "const mismatched_types: String = 7
+const invalid_annotation: MyInvalidType = \"str\""
+    );
+}
+
+#[test]
+fn const_multiple_errors_invalid_annotation() {
+    assert_module_error!(
+        "const invalid_annotation: MyInvalidType = \"str\"
+const invalid_value: String = MyInvalidValue"
+    );
+}
+
+#[test]
+fn const_multiple_errors_invalid_value() {
+    assert_module_error!(
+        "const invalid_value: String = MyInvalidValue
+const invalid_unannotated_value = [1, 2.0]"
+    );
+}
+
+#[test]
+fn const_multiple_errors_invalid_unannotated_value() {
+    assert_module_error!(
+        "const invalid_unannotated_value = [1, 2.0]
+const invalid_everything: MyInvalidType = MyInvalidValue"
+    );
+}
+
+#[test]
+fn const_multiple_errors_invalid_annotation_and_value() {
+    assert_module_error!(
+        "const invalid_everything: MyInvalidType = MyInvalidValue
+const mismatched_types: String = 7"
+    );
+}
+
+#[test]
+fn const_multiple_errors_are_local_with_annotation() {
+    assert_module_error!(
+        "const num: String = 7
+const tpl: String = #(Ok(1), MyInvalidType, 3)
+const assignment1: String = num
+const assignment2: String = tpl"
+    );
+}
+
+#[test]
+fn const_multiple_errors_are_local_with_inferred_value() {
+    assert_module_error!(
+        "const str: MyInvalidType = \"str\"
+const assignment: String = str"
+    );
+}
+
+#[test]
+fn const_multiple_errors_are_local_with_unbound_value() {
+    assert_module_error!(
+        "const lst = [1, 2.0]
+const unbound: MyInvalidType = MyInvalidType
+const assignment1: String = lst
+const assignment2: String = unbound"
+    );
+}
+
 #[test]
 fn const_usage_wrong() {
     assert_module_error!(
@@ -1643,6 +1711,32 @@ fn same_imports_multiple_times_6() {
     );
 }
 
+#[test]
+fn same_imports_multiple_times_7() {
+    assert_with_module_error!(
+        (
+            "one",
+            "
+            pub fn fn1() { 1 }
+            "
+        ),
+        (
+            "two",
+            "
+            pub fn fn2() { 1 }
+            "
+        ),
+        "
+        import one.{
+          fn1
+        } as x
+        import two.{
+          fn2
+        } as x
+        "
+    );
+}
+
 // https://github.com/gleam-lang/gleam/issues/1705
 #[test]
 fn update_multi_variant_record() {
@@ -1707,6 +1801,32 @@ pub fn main(_x: two.Thing) {
     );
 }
 
+#[test]
+fn value_imported_as_type() {
+    assert_with_module_error!(
+        (
+            "gleam/foo",
+            "pub type Bar {
+               Baz
+             }"
+        ),
+        "import gleam/foo.{type Baz}"
+    );
+}
+
+#[test]
+fn type_imported_as_value() {
+    assert_with_module_error!(
+        (
+            "gleam/foo",
+            "pub type Bar {
+               Baz
+             }"
+        ),
+        "import gleam/foo.{Bar}"
+    );
+}
+
 #[test]
 fn duplicate_module_function_arguments() {
     assert_module_error!(
@@ -1763,3 +1883,26 @@ fn list() {
 fn mismatched_list_tail() {
     assert_error!("[\"foo\", ..[1, 2]]");
 }
+
+#[test]
+fn leak_multiple_private_types() {
+    assert_module_error!(
+        "
+        type Private {
+            Private
+        }
+
+        pub fn ret_private() -> Private {
+            Private
+        }
+
+        pub fn ret_private2() -> Private {
+            Private
+        }
+
+        pub fn main() {
+            ret_private()
+        }
+        "
+    );
+}
