@@ -1203,6 +1203,127 @@ fn pattern_matching_on_tuples_doesnt_raise_a_warning() {
     );
 }
 
+#[test]
+fn pattern_matching_on_literal_empty_tuple() {
+    assert_warning!(
+        "pub fn main() {
+        case #() {
+            _ -> Nil
+        }
+      }"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_list() {
+    assert_warning!(
+        "pub fn main() {
+        case [1, 2] {
+            _ -> Nil
+        }
+      }"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_list_with_tail() {
+    assert_warning!(
+        "pub fn main() {
+        case [1, 2, ..[]] {
+            _ -> Nil
+        }
+      }"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_empty_list() {
+    assert_warning!(
+        "pub fn main() {
+        case [] {
+            _ -> Nil
+        }
+      }"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_empty_bit_array() {
+    assert_warning!(
+        "pub fn main() {
+        case <<>> {
+            _ -> Nil
+        }
+      }"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_record() {
+    assert_warning!(
+        "
+pub type Wibble { Wibble(Int) }
+pub fn main() {
+  let n = 1
+  case Wibble(n) {
+    _ -> Nil
+  }
+}"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_record_with_no_args() {
+    assert_warning!(
+        "
+pub type Wibble { Wibble }
+pub fn main() {
+  case Wibble {
+    _ -> Nil
+  }
+}"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_int() {
+    assert_warning!(
+        "
+pub type Wibble { Wibble }
+pub fn main() {
+  case 1 {
+    _ -> Nil
+  }
+}"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_float() {
+    assert_warning!(
+        "
+pub type Wibble { Wibble }
+pub fn main() {
+  case 1.0 {
+    _ -> Nil
+  }
+}"
+    );
+}
+
+#[test]
+fn pattern_matching_on_literal_string() {
+    assert_warning!(
+        "
+pub type Wibble { Wibble }
+pub fn main() {
+  case \"hello\" {
+    _ -> Nil
+  }
+}"
+    );
+}
+
 #[test]
 fn opaque_external_type_raises_a_warning() {
     assert_warning!("pub opaque type External");
@@ -1467,3 +1588,427 @@ pub type Wobble {
 }
 
 */
+
+#[test]
+fn redundant_let_assert() {
+    assert_warning!(
+        "
+pub fn main() {
+  let assert wibble = [1, 2, 3]
+  wibble
+}
+"
+    );
+}
+
+#[test]
+fn redundant_let_assert_on_custom_type() {
+    assert_warning!(
+        "
+pub type Wibble {
+    Wibble(Int, Bool)
+}
+
+pub fn main() {
+  let assert Wibble(_, bool) = Wibble(1, True)
+  bool
+}
+"
+    );
+}
+
+#[test]
+fn panic_used_as_function() {
+    assert_warning!(
+        "pub fn main() {
+          panic()
+        }"
+    );
+}
+
+#[test]
+fn panic_used_as_function_2() {
+    assert_warning!(
+        "pub fn main() {
+          panic(1)
+        }"
+    );
+}
+
+#[test]
+fn panic_used_as_function_3() {
+    assert_warning!(
+        "pub fn main() {
+          panic(1, Nil)
+        }"
+    );
+}
+
+#[test]
+fn todo_used_as_function() {
+    assert_warning!(
+        "pub fn main() {
+          todo()
+        }"
+    );
+}
+
+#[test]
+fn todo_used_as_function_2() {
+    assert_warning!(
+        "pub fn main() {
+          todo(1)
+        }"
+    );
+}
+
+#[test]
+fn todo_used_as_function_3() {
+    assert_warning!(
+        "pub fn main() {
+          todo(1, Nil)
+        }"
+    );
+}
+
+#[test]
+fn unreachable_warning_1() {
+    assert_warning!(
+        "pub fn main() {
+          panic
+          1
+        }"
+    );
+}
+
+#[test]
+fn unreachable_warning_2() {
+    assert_warning!(
+        "pub fn main() {
+          let _ = panic
+          1
+        }"
+    );
+}
+
+#[test]
+fn unreachable_warning_if_all_branches_panic() {
+    assert_warning!(
+        "pub fn main() {
+          let n = 1
+          case n {
+            0 -> panic
+            _ -> panic
+          }
+          1
+        }"
+    );
+}
+
+#[test]
+fn unreachable_warning_if_all_branches_panic_2() {
+    assert_warning!(
+        "pub fn main() {
+          let n = 1
+          case n {
+            0 -> {
+              panic
+              2
+            }
+            _ -> panic
+          }
+          1
+        }"
+    );
+}
+
+#[test]
+fn no_unreachable_warning_if_at_least_a_branch_is_reachable() {
+    assert_no_warnings!(
+        "pub fn main() {
+          let n = 1
+          case n {
+            0 -> panic
+            _ -> 1
+          }
+          1
+        }"
+    );
+}
+
+#[test]
+fn unreachable_warning_doesnt_escape_out_of_a_block_if_panic_is_not_last() {
+    assert_warning!(
+        "pub fn main() {
+          let n = {
+            panic
+            1
+          }
+          n
+        }"
+    );
+}
+
+#[test]
+fn unreachable_warning_on_following_expression_if_panic_is_last_in_a_block() {
+    assert_warning!(
+        "pub fn main() {
+          let _ = {
+            panic
+          }
+          1
+        }"
+    );
+}
+
+#[test]
+fn unreachable_function_argument_if_panic_is_argument() {
+    assert_warning!(
+        "
+        pub fn wibble(_, _) { 1 }
+        pub fn main() {
+          wibble(panic, 1)
+        }"
+    );
+}
+
+#[test]
+fn unreachable_function_call_if_panic_is_last_argument_1() {
+    assert_warning!(
+        "
+        pub fn wibble(_, _) { 1 }
+        pub fn main() {
+          wibble(1, panic)
+          1
+        }"
+    );
+}
+
+#[test]
+fn unreachable_function_call_if_panic_is_last_argument_2() {
+    assert_warning!(
+        "
+        pub fn wibble(_, _) { 1 }
+        pub fn main() {
+          wibble(1, panic)
+        }"
+    );
+}
+
+#[test]
+fn no_unreachable_warning_if_panic_comes_last_in_function_body() {
+    assert_no_warnings!(
+        "
+        pub fn wibble() { panic }
+        pub fn main() { panic }"
+    );
+}
+
+#[test]
+fn unreachable_code_for_panic_as_first_pipeline_item() {
+    assert_warning!(
+        "
+        pub fn wibble(_) { 1 }
+        pub fn main() {
+            panic |> wibble
+        }
+        "
+    );
+}
+
+#[test]
+fn panic_used_as_function_inside_pipeline() {
+    assert_warning!(
+        "
+        pub fn wibble(_) { 1 }
+        pub fn main() {
+            1 |> panic |> wibble
+        }
+        "
+    );
+}
+
+#[test]
+fn unreachable_warning_for_panic_as_last_item_of_pipe_on_next_expression() {
+    assert_warning!(
+        r#"
+        pub fn wibble(_) { 1 }
+        pub fn main() {
+            1 |> wibble |> panic
+            "unreachable"
+        }
+        "#
+    );
+}
+
+#[test]
+fn doesnt_warn_twice_for_unreachable_code_if_has_already_warned_in_a_block_1() {
+    assert_warning!(
+        r#"
+        pub fn wibble(_) { 1 }
+        pub fn main() {
+            panic
+            let _ = "unreachable" // warning here
+            panic
+            "no warning here!"
+        }
+        "#
+    );
+}
+
+#[test]
+fn doesnt_warn_twice_for_unreachable_code_if_has_already_warned_in_a_block_2() {
+    assert_warning!(
+        r#"
+        pub fn main() {
+            let _ = {
+              panic
+              1 // warning here
+            }
+            "no warning here!"
+        }
+        "#
+    );
+}
+
+#[test]
+fn unreachable_use_after_panic() {
+    assert_warning!(
+        r#"
+        pub fn wibble(_) { 1 }
+        pub fn main() {
+            panic
+            use <- wibble
+            1
+        }
+        "#
+    );
+}
+
+#[test]
+fn unreachable_code_after_case_subject_panics_1() {
+    assert_warning!(
+        r#"
+        pub fn main(a, b) {
+            case a, panic, b {
+                _, _, _ -> "no warning here!"
+            }
+        }
+        "#
+    );
+}
+
+#[test]
+fn unreachable_code_after_case_subject_panics_2() {
+    assert_warning!(
+        r#"
+        pub fn main(a, b) {
+            case a, b, panic {
+                _, _, _ -> "no warning here!"
+            }
+            "warning here!"
+        }
+        "#
+    );
+}
+
+#[test]
+fn unreachable_code_analysis_treats_anonymous_functions_independently_1() {
+    assert_no_warnings!(
+        r#"
+        pub fn main() {
+            let _ = fn() {
+              panic
+            }
+            "no warning here!"
+        }
+        "#
+    );
+}
+
+#[test]
+fn unreachable_code_analysis_treats_anonymous_functions_independently_2() {
+    assert_warning!(
+        r#"
+        pub fn main() {
+            let _ = fn() {
+              panic
+              "warning here!"
+            }
+            panic
+            "warning here!"
+        }
+        "#
+    );
+}
+
+#[test]
+fn unreachable_code_analysis_treats_anonymous_functions_independently_3() {
+    assert_warning!(
+        r#"
+        pub fn main() {
+            panic
+            let _ = "warning here!"
+            let _ = fn() {
+              panic
+              "warning here!"
+            }
+        }
+        "#
+    );
+}
+
+#[test]
+fn no_warnings_for_matches_used_like_ifs() {
+    assert_no_warnings!(
+        r#"
+    pub fn main() {
+        case True {
+          _ if True -> 1
+          _ -> 2
+        }
+    }
+        "#
+    );
+}
+
+#[test]
+fn no_warnings_for_matches_used_like_ifs_2() {
+    assert_no_warnings!(
+        r#"
+    pub fn main() {
+        case 1 {
+          _ if True -> 1
+          _ -> 2
+        }
+    }
+        "#
+    );
+}
+
+#[test]
+fn warnings_for_matches_on_literal_values_that_are_not_like_an_if_1() {
+    assert_warning!(
+        r#"
+    pub fn main() {
+        case True {
+          _ -> 1
+        }
+    }
+        "#
+    );
+}
+
+#[test]
+fn warnings_for_matches_on_literal_values_that_are_not_like_an_if_2() {
+    assert_warning!(
+        r#"
+    pub fn main() {
+        case True {
+          True -> 1
+          _ -> 2
+        }
+    }
+        "#
+    );
+}
